@@ -4,6 +4,7 @@ from collections import defaultdict, deque
 from datetime import datetime, timedelta
 import logging
 import math
+import pickle
 import random
 import socket
 from time import time
@@ -35,6 +36,7 @@ from .utils import (All, ignoring, clear_queue, get_ip, ignore_exceptions,
 logger = logging.getLogger(__name__)
 
 BANDWIDTH = 100e6
+ALLOWED_FAILURES = 3
 
 
 class Scheduler(Server):
@@ -62,7 +64,7 @@ class Scheduler(Server):
 
     Or as part of when an Executor starts up and connects to a Center::
 
-        >>> e = Executor('127.0.0.1:8787')  # doctest: +SKIP
+        >>> e = Executor('127.0.0.1:8786')  # doctest: +SKIP
         >>> e.scheduler  # doctest: +SKIP
         Scheduler(...)
 
@@ -127,6 +129,8 @@ class Scheduler(Server):
     * **loose_retrictions:** ``{key}``:
         Set of keys for which we are allow to violate restrictions (see above)
         if not valid workers are present.
+    * **suspicious_tasks:** ``{key: int}``
+        Number of times a task has been involved in a worker failure
     * **keyorder:** ``{key: tuple}``:
         A score per key that determines its priority
     * **scheduler_queues:** ``[Queues]``:
@@ -154,7 +158,7 @@ class Scheduler(Server):
     def __init__(self, center=None, loop=None,
             resource_interval=1, resource_log_size=1000,
             max_buffer_size=MAX_BUFFER_SIZE, delete_interval=500,
-            ip=None, services=None, heartbeat_interval=500, **kwargs):
+            ip=None, services=None, **kwargs):
         self.scheduler_queues = [Queue()]
         self.report_queues = []
         self.worker_streams = dict()
@@ -163,7 +167,6 @@ class Scheduler(Server):
         self.coroutines = []
         self.ip = ip or get_ip()
         self.delete_interval = delete_interval
-        self.heartbeat_interval = heartbeat_interval
         self._worker_coroutines = []
 
         self.tasks = dict()
@@ -176,7 +179,7 @@ class Scheduler(Server):
         self.nbytes = dict()
         self.ncores = dict()
         self.worker_info = defaultdict(dict)
-        self.host_info = dict()
+        self.host_info = defaultdict(dict)
         self.aliases = dict()
         self.processing = dict()
         self.rprocessing = defaultdict(set)
@@ -184,6 +187,7 @@ class Scheduler(Server):
         self.task_duration = {prefix: 0.00001 for prefix in fast_task_prefixes}
         self.restrictions = dict()
         self.loose_restrictions = set()
+        self.suspicious_tasks = defaultdict(lambda: 0)
         self.stacks = dict()
         self.waiting = dict()
         self.waiting_data = dict()
@@ -203,7 +207,6 @@ class Scheduler(Server):
         self.exceptions_blame = dict()
 
         self.loop = loop or IOLoop.current()
-        self.io_loop = self.loop
 
         self.resource_interval = resource_interval
         self.resource_log_size = resource_log_size
@@ -242,11 +245,11 @@ class Scheduler(Server):
             else:
                 port = 0
 
-            self.services[k] = v(self)
+            self.services[k] = v(self, io_loop=self.loop)
             self.services[k].listen(port)
 
         super(Scheduler, self).__init__(handlers=self.handlers,
-                max_buffer_size=max_buffer_size, **kwargs)
+                max_buffer_size=max_buffer_size, io_loop=self.loop, **kwargs)
 
     def __del__(self):
         self.close_streams()
@@ -304,7 +307,7 @@ class Scheduler(Server):
         self._delete_periodic_callback.start()
 
         if start_queues:
-            self.handle_queues(self.scheduler_queues[0], None)
+            self.loop.add_callback(self.handle_queues, self.scheduler_queues[0], None)
 
         for cor in self.coroutines:
             if cor.done():
@@ -329,21 +332,25 @@ class Scheduler(Server):
             yield All(self.coroutines)
 
     def close_streams(self):
-        for r in self._rpcs.values():
-            r.close_streams()
+        with ignoring(AttributeError):
+            for r in self._rpcs.values():
+                r.close_streams()
         for stream in self.streams.values():
             stream.stream.close()
 
     @gen.coroutine
-    def close(self, stream=None):
+    def close(self, stream=None, fast=False):
         """ Send cleanup signal to all coroutines then wait until finished
 
         See Also
         --------
         Scheduler.cleanup
         """
+        for service in self.services.values():
+            service.stop()
         yield self.cleanup()
-        yield self.finished()
+        if not fast:
+            yield self.finished()
         self.close_streams()
         self.status = 'closed'
         self.stop()
@@ -357,7 +364,7 @@ class Scheduler(Server):
         self.status = 'closing'
         logger.debug("Cleaning up coroutines")
 
-        for w, bstream in self.worker_streams.items():
+        for w, bstream in list(self.worker_streams.items()):
             with ignoring(AttributeError):
                 yield bstream.close(ignore_closed=True)
 
@@ -750,6 +757,17 @@ class Scheduler(Server):
             if key not in self.dependencies:
                 self.dependencies[key] = set()
 
+    def mark_task_killed_worker(self, key=None, worker=None):
+        """ Mark that is likely killing workers """
+        self.exceptions[key] = pickle.dumps(KilledWorker(key, worker))
+        self.tracebacks[key] = None
+        self.mark_failed(key, key)
+        for plugin in self.plugins[:]:
+            try:
+                plugin.task_erred(self, key=key, worker=worker,
+                        exception=exception, traceback=traceback, **kwargs)
+            except Exception as e:
+                logger.exception(e)
 
     def mark_task_erred(self, key=None, worker=None,
                         exception=None, traceback=None, **kwargs):
@@ -765,7 +783,6 @@ class Scheduler(Server):
             self.tracebacks[key] = traceback
             self.mark_failed(key, key)
             self.maybe_idle.add(worker)
-            # self.ensure_occupied(worker)
             for plugin in self.plugins[:]:
                 try:
                     plugin.task_erred(self, key=key, worker=worker,
@@ -897,25 +914,6 @@ class Scheduler(Server):
         logger.debug('\n\nwaiting: %s\n\nstacks: %s\n\nprocessing: %s\n\n',
                      self.waiting, self.stacks, self.processing)
 
-    def _restart_heartbeat(self, host):
-        """ Restart heartbeat periodic callback from among active ports """
-        d = self.host_info[host]
-
-        if 'heartbeat-port' in d and d['heartbeat-port'] in d['ports']:
-            return
-
-        if 'heartbeat' in d:
-            d['heartbeat'].stop()
-
-        port = first(d['ports'])
-
-        pc = PeriodicCallback(callback=lambda: self.heartbeat(host),
-                              callback_time=self.heartbeat_interval,
-                              io_loop=self.loop)
-        self.loop.add_callback(pc.start)
-        d['heartbeat'] = pc
-        d['heartbeat-port'] = port
-
     def remove_worker(self, stream=None, address=None):
         """ Mark that a worker no longer seems responsive
 
@@ -940,24 +938,28 @@ class Scheduler(Server):
 
             if not self.host_info[host]['ports']:
                 del self.host_info[host]
-            else:
-                self._restart_heartbeat(host)
 
             del self.worker_streams[address]
             del self.ncores[address]
             del self.aliases[self.worker_info[address]['name']]
             del self.worker_info[address]
+            if address in self.maybe_idle:
+                self.maybe_idle.remove(address)
             if address in self.idle:
                 self.idle.remove(address)
             if address in self.saturated:
                 self.saturated.remove(address)
 
             in_flight = set(self.processing.pop(address))
-            for k in in_flight:
+            for k in list(in_flight):
+                self.suspicious_tasks[k] += 1
                 s = self.rprocessing[k]
                 s.remove(address)
                 if not s:
                     del self.rprocessing[k]
+                if self.suspicious_tasks[k] > ALLOWED_FAILURES:
+                    self.mark_task_killed_worker(key=k, worker=address)
+                    in_flight.remove(k)
             in_flight |= set(self.stacks.pop(address))
             del self.occupancy[address]
             in_flight = {k for k in in_flight if k in self.tasks}
@@ -985,37 +987,52 @@ class Scheduler(Server):
                 self.ensure_in_play(key)
 
             if not self.stacks:
-                logger.critical("Lost all workers")
+                logger.info("Lost all workers")
 
             self.ensure_idle_ready()
 
             return 'OK'
 
     def add_worker(self, stream=None, address=None, keys=(), ncores=None,
-                   name=None, coerce_address=True, nbytes=None, **info):
+                   name=None, coerce_address=True, nbytes=None, now=None,
+                   host_info=None, **info):
         with log_errors():
+            local_now = time()
+            now = now or time()
+            info = info or {}
+            host_info = host_info or {}
             if coerce_address:
                 address = self.coerce_address(address)
+                host, port = address.split(':')
+                self.host_info[host]['last-seen'] = local_now
+
+            if info:
+                self.worker_info[address].update(info)
+
+            if host_info:
+                self.host_info[host].update(host_info)
+
+            delay = time() - now
+            self.worker_info[address]['time-delay'] = delay
+            self.worker_info[address]['last-seen'] = time()
+
+            if address in self.ncores:
+                return 'OK'
+
             name = name or address
             if name in self.aliases:
                 return 'name taken, %s' % name
 
             if coerce_address:
-                host, port = self.coerce_address(address).split(':')
-                if host not in self.host_info:
-                    self.host_info[host] = {'ports': set(), 'cores': 0}
+                if 'ports' not in self.host_info[host]:
+                    self.host_info[host].update({'ports': set(), 'cores': 0})
 
                 self.host_info[host]['ports'].add(port)
                 self.host_info[host]['cores'] += ncores
-                self.loop.add_callback(self.heartbeat, host)
-                self._restart_heartbeat(host)
 
             self.ncores[address] = ncores
-
             self.aliases[name] = address
-
-            info['name'] = name
-            self.worker_info[address] = info
+            self.worker_info[address]['name'] = name
 
             if address not in self.processing:
                 self.has_what[address] = set()
@@ -1252,6 +1269,8 @@ class Scheduler(Server):
                     del self.waiting[key]
                 if key in self.waiting_data:
                     del self.waiting_data[key]
+                if key in self.suspicious_tasks:
+                    del self.suspicious_tasks[key]
                 for w in self.rprocessing.pop(key, ()):
                     self.occupancy[w] -= self.processing[w].pop(key)
 
@@ -1544,41 +1563,6 @@ class Scheduler(Server):
             if key in self.nbytes:
                 del self.nbytes[key]
 
-    @gen.coroutine
-    def heartbeat(self, host):
-        port = self.host_info[host]['heartbeat-port']
-        start_time = time()
-        start = default_timer()
-
-        try:
-            d = yield gen.with_timeout(timedelta(seconds=1),
-                                       self.rpc(ip=host, port=port).health(),
-                                       io_loop=self.loop)
-        except gen.TimeoutError:
-            logger.warn("Heartbeat failed for %s", host)
-            return
-        except Exception as e:
-            logger.exception(e)
-            return
-
-        end = default_timer()
-        end_time = time()
-        last_seen = datetime.now()
-
-        d['latency'] = end - start
-        d['last-seen'] = last_seen
-
-        delay = (end_time + start_time) / 2 - d['time']
-        try:
-            avg_delay = self.host_info[host]['time-delay']
-            avg_delay = (0.90 * avg_delay + 0.10 * delay)
-        except KeyError:
-            avg_delay = delay
-
-        d['time-delay'] = delay
-
-        self.host_info[host].update(d)
-
     @gen.coroutine
     def scatter(self, stream=None, data=None, workers=None, client=None,
             broadcast=False):
@@ -2133,3 +2117,7 @@ _round_robin = [0]
 
 
 fast_task_prefixes = {'rechunk-split'}
+
+
+class KilledWorker(Exception):
+    pass
